package com.cultural.heritage.controller.good;


import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.cultural.heritage.annotation.AuthCheck;
import com.cultural.heritage.common.BaseResponse;
import com.cultural.heritage.common.ErrorCode;
import com.cultural.heritage.common.ResultUtils;
import com.cultural.heritage.constant.CouponStatus;
import com.cultural.heritage.constant.UserConstant;
import com.cultural.heritage.exception.BusinessException;
import com.cultural.heritage.exception.ThrowUtils;
import com.cultural.heritage.model.dto.CommonRequest;
import com.cultural.heritage.model.dto.coupon.CouponAddRequest;
import com.cultural.heritage.model.dto.coupon.CouponQueryRequest;
import com.cultural.heritage.model.dto.coupon.CouponShelvesUpdateRequest;
import com.cultural.heritage.model.dto.coupon.CouponUpdateRequest;
import com.cultural.heritage.model.dto.exchange.ExchangeAddRequest;
import com.cultural.heritage.model.entity.Coupon;
import com.cultural.heritage.model.entity.Exchange;
import com.cultural.heritage.model.entity.User;
import com.cultural.heritage.model.vo.coupon.CouponVO;
import com.cultural.heritage.model.vo.exchange.ExchangeVO;
import com.cultural.heritage.service.good.CouponService;
import com.cultural.heritage.service.good.ExchangeService;
import com.cultural.heritage.service.user.UserService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.annotation.Resource;
import jakarta.servlet.http.HttpServletRequest;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.stream.Collectors;


@RestController
@RequestMapping("/coupon")
@Slf4j
@Tag(name = "优惠券管理模块")
public class CouponController {

    @Resource
    private CouponService couponService;


    @Resource
    private UserService userService;


    @Resource
    private ExchangeService exchangeService;


    /**
     * 添加优惠券
     * @param couponAddRequest  优惠券添加请求体
     * @return  是否添加成功
     */
    @PostMapping("/add")
    @Operation(summary = "Web端管理员添加优惠券", description = "参数：优惠券添加请求体，权限：管理员(admin, boss)，方法名：addCoupon")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Boolean> addCoupon(@RequestBody CouponAddRequest couponAddRequest) {
        if (couponAddRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Coupon coupon = new Coupon();
        BeanUtils.copyProperties(couponAddRequest, coupon);
        // 校验
        couponService.validCoupon(coupon, false);
        boolean result = couponService.save(coupon);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR);
        return ResultUtils.success(true);
    }



    /**
     * 更新优惠券
     * @param couponUpdateRequest   优惠券更新请求体
     * @return  是否更新成功
     */
    @PostMapping("/update")
    @Operation(summary = "Web端管理员更新优惠券", description = "参数：优惠券更新请求体，权限：管理员(admin, boss)，方法名：updateCoupon")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Boolean> updateCoupon(@RequestBody CouponUpdateRequest couponUpdateRequest) {
        if (couponUpdateRequest == null || couponUpdateRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Coupon coupon = new Coupon();
        BeanUtils.copyProperties(couponUpdateRequest, coupon);
        // 校验
        couponService.validCoupon(coupon, true);
        boolean result = couponService.updateById(coupon);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR);
        return ResultUtils.success(true);
    }



    /**
     * 删除优惠券
     * @param couponDeleteRequest   优惠券删除请求体
     * @return  是否删除成功
     */
    @PostMapping("/delete")
    @Operation(summary = "Web端管理员删除优惠券", description = "参数：优惠券删除请求体，权限：管理员(admin, boss)，方法名：deleteCoupon")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Boolean> deleteCoupon(@RequestBody CommonRequest couponDeleteRequest) {
        if (couponDeleteRequest == null || couponDeleteRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Long id = couponDeleteRequest.getId();
        boolean result = couponService.removeById(id);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR);
        return ResultUtils.success(true);
    }


    /**
     * 查询优惠券列表
     * @param couponQueryRequest 优惠券查询请求体
     * @return 优惠券列表
     */
    @PostMapping("/list/page")
    @Operation(summary = "Web端管理员分页查询优惠券", description = "参数：优惠券查询请求体, 排序字段 sortField: conditionAmount requirePoints totalNum residueNum limitNum id，权限：管理员(admin, boss)，方法名：listCouponVOByPage")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Page<Coupon>> listCouponVOByPage(@RequestBody CouponQueryRequest couponQueryRequest) {
        if (couponQueryRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        long current = couponQueryRequest.getCurrent();
        long pageSize = couponQueryRequest.getPageSize();
        Page<Coupon> page = couponService.page(new Page<>(current, pageSize),
                couponService.getQueryWrapper(couponQueryRequest));
        return ResultUtils.success(page);
    }


    /**
     * 用户删除兑换记录
     * @param commonRequest  兑换记录删除请求体
     * @return  是否兑换成功
     */
    @PostMapping("/delRecord")
    @Operation(summary = "小程序端用户删除兑换记录", description = "参数：兑换记录删除请求体，权限：管理员(admin, boss)，方法名：deleteExchangeRecord")
    public BaseResponse<Boolean> deleteExchangeRecord(@RequestBody CommonRequest commonRequest, HttpServletRequest request) {
        if (commonRequest == null || commonRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        userService.getLoginUser(request);
        Long id = commonRequest.getId();
        boolean result = exchangeService.removeById(id);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR);
        return ResultUtils.success(true, "删除兑换记录成功");
    }


    /**
     * 小程序端用户积分兑换优惠券
     * @param exchangeAddRequest 兑换记录添加请求体
     * @return
     */
    @PostMapping("/exchange")
    @Operation(summary = "小程序端用户积分兑换优惠券", description = "参数：兑换记录添加请求体，权限：所有人，方法名：pointsExchangeCoupon")
    @Transactional(rollbackFor = Exception.class)
    public BaseResponse<Boolean> pointsExchangeCoupon(@RequestBody ExchangeAddRequest exchangeAddRequest, HttpServletRequest request) {
        // 获取当前用户信息
        User loginUser = userService.getLoginUser(request);
        Long userId = loginUser.getId();
        Integer points = loginUser.getPoints();

        // 获取当前优惠券信息
        Long couponId = exchangeAddRequest.getCouponId();
        Integer quantity = exchangeAddRequest.getQuantity();
        Coupon coupon = couponService.getById(couponId);
        ThrowUtils.throwIf(coupon == null, ErrorCode.OPERATION_ERROR, "优惠券不存在");
        Integer limitNum = coupon.getLimitNum();
        Integer residueNum = coupon.getResidueNum();

        // 判断优惠券库存是否足够
        if (quantity > residueNum) {
            throw new BusinessException(ErrorCode.OPERATION_ERROR, "库存不足");
        }

        // 判断用户限领量是否已达标
        QueryWrapper<Exchange> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("userId", userId).eq("couponId", couponId);
        List<Exchange> exchangeList = exchangeService.list(queryWrapper);
        int sum = 0;
        for (int i = 0; i < exchangeList.size(); i ++ ) {
            Exchange exchange = exchangeList.get(i);
            sum += exchange.getQuantity();
        }
        if (sum + quantity > limitNum) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "当前优惠券的限领量已达标");
        }

        // 判断用户积分是否充足
        Integer requirePoints = coupon.getRequirePoints();
        if (points < quantity * requirePoints) throw new BusinessException(ErrorCode.OPERATION_ERROR, "积分不足");

        // 更新优惠券库存量
        coupon.setResidueNum(coupon.getResidueNum() - quantity);
        boolean result = couponService.updateById(coupon);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR, "更新优惠券库存失败");

        // 插入一条兑换记录
        CouponVO couponVO = new CouponVO();
        BeanUtils.copyProperties(coupon, couponVO);
        Exchange exchange = new Exchange();
        exchange.setUserId(userId);
        exchange.setQuantity(quantity);
        exchange.setRestNum(quantity);
        exchange.setRequirePoints(requirePoints);
        exchange.setCouponId(couponId);
        exchange.setCouponDetail(couponVO);
        boolean save = exchangeService.save(exchange);
        ThrowUtils.throwIf(!save, ErrorCode.OPERATION_ERROR, "兑换记录插入失败");

        // 更新用户积分
        loginUser.setPoints(loginUser.getPoints() - quantity * requirePoints);
        boolean isSuccess = userService.updateById(loginUser);
        ThrowUtils.throwIf(!isSuccess, ErrorCode.OPERATION_ERROR, "更新用户积分信息失败");

        return ResultUtils.success(true, "兑换成功");
    }





    /**
     * 小程序端用户查看优惠券
     * @return 兑换记录列表
     */
    @GetMapping("/list/my")
    @Operation(summary = "小程序端用户查看优惠券", description = "参数：无，权限：所有人，方法名：pointsExchangeCoupon")
    public BaseResponse<List<ExchangeVO>> listMyCoupon(HttpServletRequest request) {
        User loginUser = userService.getLoginUser(request);
        Long userId = loginUser.getId();
        QueryWrapper<Exchange> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("userId", userId);
        List<Exchange> exchangeList = exchangeService.list(queryWrapper);
        List<ExchangeVO> exchangeVOS = exchangeList.stream().map(exchange -> {
            ExchangeVO exchangeVO = new ExchangeVO();
            BeanUtils.copyProperties(exchange, exchangeVO);
            String startTime = exchange.getCouponDetail().getStartTime();
            String endTime = exchange.getCouponDetail().getEndTime();
            DateTime start = DateUtil.parse(startTime);
            DateTime end = DateUtil.parse(endTime);
            DateTime current = DateUtil.date();

            boolean isBefore = current.isBefore(start);
            boolean isAfter = current.isAfter(end);
            boolean isInRange = current.isAfterOrEquals(start) && current.isBeforeOrEquals(end);

            if (isBefore) exchangeVO.setCouponStatus(CouponStatus.IS_NOT_ENABLED);
            if (isAfter) exchangeVO.setCouponStatus(CouponStatus.IS_OVERDUE);
            if (isInRange) exchangeVO.setCouponStatus(CouponStatus.IS_ENABLED);
            return exchangeVO;
        }).toList();
        List<ExchangeVO> exchangeVOList = exchangeVOS.stream().filter(exchangeVO -> exchangeVO.getRestNum() != 0)
                .collect(Collectors.toList());
        return ResultUtils.success(exchangeVOList);
    }





    /**
     * Web端管理员上(下)架优惠券
     * @param couponShelvesUpdateRequest 优惠券上架状态更新请求体
     * @return 是否更新成功
     */
    @PostMapping("/shelves")
    @Operation(summary = "Web端管理员上(下)架优惠券", description = "参数：优惠券上架状态更新请求体，权限：管理员(admin, boss)，方法名：deleteExchangeRecord")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Boolean> updateCouponShelvesStatus(@RequestBody CouponShelvesUpdateRequest couponShelvesUpdateRequest) {
        if (couponShelvesUpdateRequest == null || couponShelvesUpdateRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Long id = couponShelvesUpdateRequest.getId();
        Integer isShelves = couponShelvesUpdateRequest.getIsShelves();
        UpdateWrapper<Coupon> updateWrapper = new UpdateWrapper<>();
        updateWrapper.eq("id", id);
        updateWrapper.set("isShelves", isShelves);
        boolean update = couponService.update(updateWrapper);
        ThrowUtils.throwIf(!update, ErrorCode.OPERATION_ERROR, "上架状态更新失败");
        return ResultUtils.success(true);
    }





}
