package com.cultural.heritage.controller.wx;


import com.cultural.heritage.common.BaseResponse;
import com.cultural.heritage.common.ErrorCode;
import com.cultural.heritage.common.ResultUtils;
import com.cultural.heritage.constant.OrderStatusConstant;
import com.cultural.heritage.exception.BusinessException;
import com.cultural.heritage.exception.ThrowUtils;
import com.cultural.heritage.model.dto.CommonRequest;
import com.cultural.heritage.model.entity.Order;
import com.cultural.heritage.model.entity.User;
import com.cultural.heritage.service.order.OrderService;
import com.cultural.heritage.service.user.UserService;
import com.cultural.heritage.service.wxpay.WeChatService;
import com.wechat.pay.java.service.payments.jsapi.model.PrepayWithRequestPaymentResponse;
import com.wechat.pay.java.service.payments.model.Transaction;
import com.wechat.pay.java.service.refund.model.Refund;
import com.wechat.pay.java.service.refund.model.RefundNotification;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.annotation.Resource;
import jakarta.servlet.http.HttpServletRequest;
import lombok.extern.slf4j.Slf4j;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import java.io.IOException;

/**
 * 微信小程序相关接口
 **/
@Slf4j
@RestController
@Tag(name = "微信支付接口")
@RequestMapping("/wechat")
public class WeChatPayController {

    @Resource
    private UserService userService;

    @Resource
    private OrderService ordersService;

    @Resource
    private WeChatService weChatService;


    /**
     * JSAPI 下单
     */
    @PostMapping("/payment/create")
    public BaseResponse<PrepayWithRequestPaymentResponse> createPayment(@RequestBody CommonRequest commonRequest, HttpServletRequest request) {
        User loginUser = userService.getLoginUser(request);
        String miniOpenId = loginUser.getMiniOpenId();
        ThrowUtils.throwIf(miniOpenId == null, ErrorCode.NOT_FOUND_ERROR, "不是小程序用户");
        Long orderId = commonRequest.getId();
        Order order = ordersService.getById(orderId);
        ThrowUtils.throwIf(order == null, ErrorCode.NOT_FOUND_ERROR, "订单不存在");
        ThrowUtils.throwIf(!order.getOrderStatus().equals(OrderStatusConstant.PENDING_PAYMENT), ErrorCode.OPERATION_ERROR, "订单状态错误");
        if (!loginUser.getId().equals(order.getUserId())) {
            throw new BusinessException(ErrorCode.NO_AUTH_ERROR, "你不是该订单用户！");
        }
        PrepayWithRequestPaymentResponse response = weChatService.createPayment(String.valueOf(orderId), miniOpenId, order.getTotalAmount());
        return ResultUtils.success(response);
    }

    /**
     * JSAPI 下单回调
     */
    @PostMapping("/payment/callback")
    @Transactional(rollbackFor = Exception.class)
    public synchronized BaseResponse<Boolean> callbackPayment(HttpServletRequest request) throws IOException {
        // 获取下单信息
        Transaction transaction = weChatService.getTransactionInfo(request);
        System.out.println("下单信息:" + transaction);
        // 支付回调
        boolean result = weChatService.paymentCallback(transaction);
        ThrowUtils.throwIf(!result, ErrorCode.SYSTEM_ERROR, "微信支付回调失败");
        return ResultUtils.success(true);
    }

    /**
     * 退款（仅管理员和商家）
     */
    @PostMapping("/refund/create")
//    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Refund> createRefund(@RequestBody CommonRequest commonRequest) {
        Long orderId = commonRequest.getId();
        Order order = ordersService.getById(orderId);
        ThrowUtils.throwIf(order == null, ErrorCode.NOT_FOUND_ERROR, "订单不存在");
        Refund refund = weChatService.refundPayment(String.valueOf(orderId), order.getTotalAmount());
        return ResultUtils.success(refund);
    }

    /**
     * 退款回调
     */
    @PostMapping("/refund/callback")
    public BaseResponse<Boolean> callbackRefund(HttpServletRequest request) {
        // 获取退款信息
        RefundNotification refundNotification = weChatService.getRefundInfo(request);
        // 退款回调
        boolean result = weChatService.refundCallback(refundNotification);
        ThrowUtils.throwIf(!result, ErrorCode.SYSTEM_ERROR, "退款回调失败");
        return ResultUtils.success(true);
    }

    /**
     * 发送订阅消息
     */
    @GetMapping("/refund/callback")
    public BaseResponse<Boolean> testSendMessage() {
        String miniOpenId = "oydCP7TGAWUqN_d0g3y6bbQkbe2I";
        String templateId = "AJqdU29ETU1kzBGzXaxA-OVs5AvGY1s4xaSJY1JNaiE";
        boolean subscribeMessage = weChatService.sendSubscribeMessage(miniOpenId, templateId);
        return ResultUtils.success(subscribeMessage);
    }

}
